@@ -14,7 +14,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; see the file COPYING. If not, write to the
 # Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+from multiprocessing import Pool
+from optparse import OptionParser
 from subprocess import PIPE, Popen
+from cStringIO import StringIO
 import ConfigParser
 from itertools import chain
 import os
@@ -31,16 +34,14 @@ from zc.buildout.easy_install import Installer, MissingDistribution
 from distutils.errors import DistutilsError
 
 from collective.releaser import base
-from collective.releaser.base import savewd
+from collective.releaser import packet
+from collective.releaser.base import savewd, display, display_step
 
 join = os.path.join
 
 extract_path = re.compile(r'^((?:htt(?:p|ps)|svn|file)://)(.*?)/?$')
 is_svn = re.compile(r'^((?:htt(?:p|ps)|svn|file)://)(.*?)/?$')
 
-def _log(msg):
-    print msg
-
 def _extract_url(url):
     extracted = extract_path.findall(url)
     protocol = extracted[0][0]
@@ -48,7 +49,9 @@ def _extract_url(url):
     return protocol, path
 
 @savewd
-def make_release(version=None):
+def make_release(version=None, release=None,
+                 prod_cfg='prod.cfg', dev_cfg='dev.cfg',
+                 verbose=False):
     """this is called when a release is made over a buildout"""
     if not base.has_svn_support():
         raise base.ReleaseError('You need Subversion to create a release tag')
@@ -60,9 +63,50 @@ def make_release(version=None):
 
     # next, let's create a branch for the release
     # after we asked the user what is the version to release
+    parser = OptionParser()
+    parser.add_option("-v", "--version", dest="version",
+                      help="Version to release",
+                      default=version, metavar="VERSION")
+    parser.add_option("-d", "--dev-config", dest="dev_cfg",
+            help="A .cfg with develop egg to include in the tarball. Default: %s" % dev_cfg,
+                      default=dev_cfg, metavar="DEV_CFG")
+    parser.add_option("-t", "--target-config", dest="prod_cfg",
+                      help="A .cfg to use for the release. Default: %s" % prod_cfg,
+                      default=prod_cfg, metavar="TARGET_CFG")
+    parser.add_option("--no-archive", dest="release",
+                      help="Only tag the project. Don't create an archive.",
+                      default=True, action="store_false")
+    parser.add_option("--verbose", dest="verbose",
+                      help="More print",
+                      action='store_true', default=verbose)
+    options, args = parser.parse_args()
+
+    dev_cfg = options.dev_cfg
+    prod_cfg = options.prod_cfg
+
+    verbose = options.verbose
+
+    if version is None:
+        version = options.version
+
+    if release is None:
+        release = options.release
+
     if version is None:
         version = raw_input('What version you are releasing ? ')
 
+    if release and (not os.path.isfile(dev_cfg) or not os.path.isfile(prod_cfg)):
+        parser.error('Bad config files')
+
+    # get full path to develop eggs
+    if os.path.isfile(dev_cfg):
+        buildout_dev = Buildout(dev_cfg, [])
+        develop_eggs = buildout_dev['buildout'].get('develop')
+        develop_eggs = [os.path.abspath(e.strip()) for e in develop_eggs.split('\n')]
+        develop_eggs = [e for e in develop_eggs if e]
+    else:
+        develop_eggs = []
+
     # were are we ?
     url = base.get_svn_url()
 
@@ -84,7 +128,7 @@ def make_release(version=None):
     base.svn_copy(url, release, 'creating %s release for project' % version)
 
     # let's get it to add a version.txt file in the buildout root
-    rep = mkdtemp()
+    rep = mkdtemp('project_release')
     try:
         base.svn_checkout(release, rep)
         os.chdir(rep)
@@ -92,8 +136,41 @@ def make_release(version=None):
         open(version_file, 'w').write(version)
         base.svn_add(version_file)
         msg = 'Added version file to buildout.'
-        _log(msg)
-        base.svn_commit(msg)
+        display_step(msg, verbose)
+
+        eggs = {}
+        if release and develop_eggs:
+            # release eggs
+            set_option(dev_cfg, 'buildout', 'develop', '\n'.join(develop_eggs))
+            eggs = release_eggs(version=version, verbose=verbose,
+                                prod_cfg=prod_cfg, dev_cfg=dev_cfg)
+
+            # restore dev.cfg
+            os.remove(dev_cfg)
+            base.svn_up(dev_cfg)
+
+            # fix buildout.cfg
+            if prod_cfg != 'buildout.cfg':
+                set_option('buildout.cfg', 'buildout', 'extends', prod_cfg)
+            for egg, ver in eggs.items():
+                set_option('buildout.cfg', 'versions', egg, ver)
+            eggs_versions = ', '.join(sorted(['%s=%s' % egg for egg in eggs.items()]))
+            msg += ' Set versions to %s' % eggs_versions
+
+            base.svn_commit(msg)
+
+            if eggs:
+                # archive
+                py_version = 'py%s.%s' % sys.version_info[0:2]
+                archive = join(dir, 'release-eggs-%s-%s.tar.gz' % (version, py_version))
+                eggs = project_eggs('buildout.cfg', archive, filter_eggs=eggs.keys(),
+                                    verbose=verbose)
+                display('Archive created at %s with %s' % (archive, ' '.join(eggs)))
+            else:
+                display('There is no eggs to release')
+        else:
+            base.svn_commit(msg)
+
     finally:
         shutil.rmtree(rep, ignore_errors=True)
 
@@ -146,7 +223,7 @@ def diff_releases(old=None, new=None, result=None):
 
     tmp = mkdtemp()
 
-    _log('Selecting files')
+    display('Selecting files')
     for f in new_tarball.getmembers():
         if f.name in old_files:
             if f.isfile():
@@ -160,12 +237,12 @@ def diff_releases(old=None, new=None, result=None):
         new_tarball.extract(f, tmp)
 
     # now writing the diff tarball
-    _log('Writing archive')
+    display('Writing archive')
     archive_contents(result, tmp)
     # the tarball is create in the folder up `tmp`, let's move it
     topdir = os.path.split(tmp)[0]
     shutil.move(join(topdir, result), join(working_dir, result))
-    _log('Diff done.')
+    display('Diff done.')
 
 @savewd
 def archive_contents(archive, location, exclude=None, source=True):
@@ -240,9 +317,9 @@ def _easy_install():
 
     if target in found:
         return target
-    for found in found:
-        if os.path.exists(found):
-            return found
+    for binary in found:
+        if os.path.exists(binary):
+            return binary
     return 'easy_install'
 
 def _python(home_dir=""):
@@ -250,15 +327,19 @@ def _python(home_dir=""):
     if sys.platform == 'win32':
         found = glob(join(home_dir, 'Scripts', 'python*'))
         target = join(home_dir, 'Scripts', 'python.exe')
+    elif sys.platform == 'darwin':
+        found = glob(join(home_dir, 'bin', 'Python*'))
+        found.extend(glob(join(home_dir, 'bin', 'python*')))
+        target = join(home_dir, 'bin', 'python')
     else:
         found = glob(join(home_dir, 'bin', 'python*'))
         target = join(home_dir, 'bin', 'python')
 
     if target in found:
         return target
-    for found in found:
-        if os.path.exists(found):
-            return found
+    for binary in found:
+        if os.path.exists(binary):
+            return binary
     return 'python'
 
 def _set_dynlibs(root):
@@ -279,7 +360,7 @@ def _make_python(location='.'):
     # to make sure we don't loose eggs
     old = sys.argv
     try:
-        sys.argv = ['project_deploy', '--no-site-packages', location]
+        sys.argv = ['project_deploy', '--no-site-packages', location, '--quiet']
         from virtualenv import main
         try:
             def after_install(options, home_dir):
@@ -312,24 +393,56 @@ def _make_python(location='.'):
     else:
         return join(location, 'bin', 'python')
 
+def runwait(command, env={}, debug=False):
+    if env:
+        p = Popen(command, env=env, shell=True,
+                  stdout=PIPE, stderr=PIPE)
+    else:
+        p = Popen(command, shell=True,
+                  stdout=PIPE, stderr=PIPE)
+    stdout = StringIO()
+    output = 'x'
+    while output:
+        output = p.stdout.readline()
+        if debug:
+            sys.stdout.write(output)
+            sys.stdout.flush()
+        else:
+            stdout.write(output)
+    stdout.seek(0)
+    return p.wait(), stdout.read(), p.stderr.read()
+
 @savewd
 def deploy_release(path=None, target=None, archiving='full'):
     """deploy a release in-place"""
+    buildout_args = '-v'
     if path is None:
-        if len(sys.argv) < 2:
-            print 'usage : project_deploy [http://to/your/buildout/]config.cfg'
-            sys.exit(1)
-        path = sys.argv[1]
+        usage = '%prog [options] [http://to/your/buildout/]config.cfg]'
+        parser = OptionParser(usage=usage)
+        parser.add_option("-v", "--verbose", dest="verbose",
+                          help="increase verbosity",
+                          default=False, action="store_true")
+        parser.add_option("-D", "--debug", dest="debug",
+                          help="enable debug mode",
+                          default=False, action="store_true")
+        options, args = parser.parse_args()
+        if len(args) != 1:
+            parser.parse_args(['-h'])
+        path = args[0]
+        if options.verbose:
+            buildout_args += 'vv'
+        if options.debug:
+            buildout_args += ' -D'
 
     location, filename = parse_url(path)
     folder = location.split('/')[-1]
 
     if os.path.isfile(path):
-        _log("Using local configuration file %s" % filename)
+        display("Using local configuration file %s" % filename)
         target, filename = os.path.split(path)
         release_name = filename.split('.')[0]
     else:
-        _log("Using network file %s with url %s" % (filename, location))
+        display("Using network file %s with url %s" % (filename, location))
         release_name = '%s-%s' % (folder, filename.split('.')[0])
 
     if not target:
@@ -357,9 +470,9 @@ def deploy_release(path=None, target=None, archiving='full'):
         # let's get the buildout
         if is_svn.search(path) is None:
             raise base.ReleaseError('%s is not a svn url' % path)
-        _log(base.system('svn co %s .' % location))
+        display(base.system('svn co %s .' % location))
 
-    _log('Using local directory %s with %s' % (target, filename))
+    display('Using local directory %s with %s' % (target, filename))
 
     # let's generate a virtualenv python
     # to make sure we don't loose eggs
@@ -372,11 +485,11 @@ def deploy_release(path=None, target=None, archiving='full'):
     # now we can run bootstrap.py
     python = _python()
 
-    _log(base.system('%s bootstrap.py' % python))
+    display(base.system('%s bootstrap.py -c %s' % (python, filename)))
 
     # let's try to upgrade setuptools and zc.buildout
-    _log(base.system('%s -U setuptools' % _easy_install()))
-    _log(base.system('%s -U zc.buildout' % _easy_install()))
+    display(base.system('%s -U setuptools' % _easy_install()))
+    display(base.system('%s -U zc.buildout' % _easy_install()))
 
     # then bin/buildout -v
     binary = join('bin', 'buildout')
@@ -390,34 +503,29 @@ def deploy_release(path=None, target=None, archiving='full'):
     #    raise base.ReleaseError(("You need to add 'download-cache = downloads'"
     #                             "' in the [buildout] section"))
 
-    _log('Checking binary.')
-    buildout_cmd = '%s -c %s -v' % (binary, filename)
-    _log('Calling %s' % buildout_cmd)
-
-    def runwait(command):
-        p = Popen(command, shell=True,
-                  stdout=PIPE, stderr=PIPE)
-        output = 'x'
-        while output:
-            output = p.stdout.readline()
-            sys.stdout.write(output)
-            sys.stdout.flush()
-        errors = p.stderr.read()
-        return p.wait(), errors
+    display('Checking binary.')
+    buildout_cmd = '%s %s -c %s' % (binary, buildout_args, filename)
+    display('Calling %s' % buildout_cmd)
 
     try:
-        exit_code, errors = runwait(buildout_cmd)
+        exit_code, output, errors = runwait(buildout_cmd)
         if exit_code != 0:
-            _log(errors)
+            display(errors)
             raise
+        else:
+            sys.stdout.write(output)
+            sys.stdout.flush()
     except:
         # second chance (re-runned buildouts)
-        exit_code, errors = runwait(buildout_cmd)
+        exit_code, output, errors = runwait(buildout_cmd)
         if exit_code != 0:
-            _log(errors)
+            display(errors)
             raise base.ReleaseError('Fix your buildout')
+        else:
+            sys.stdout.write(output)
+            sys.stdout.flush()
 
-    _log('%s ok.' % binary)
+    display('%s ok.' % binary)
 
     # archiving process: changes buildout.cfg and create a tarball
     old_content = open(filename).read()
@@ -427,13 +535,13 @@ def deploy_release(path=None, target=None, archiving='full'):
         set_option(buildout, 'buildout', 'install-from-cache', 'true')
         set_option(buildout, 'buildout', 'offline', 'true')
         # let's generate the MD5 stamp that will go in the archive
-        _log('Generating MD5...')
+        display('Generating MD5...')
         open(join(root, 'MD5'), 'w').write(build_md5(root))
 
         # archiving:
         #  - a full archive
         #  - an diffed archived
-        _log('Archiving %s.' % target)
+        display('Archiving %s.' % target)
         if archiving == 'none':
             return
         version_txt = join(root, 'version.txt')
@@ -443,7 +551,7 @@ def deploy_release(path=None, target=None, archiving='full'):
         else:
             archive_name = 'release-%s.tgz' % release_name
         archive_contents(archive_name, '.', exclude)
-        _log('%s ok.' % archive_name)
+        display('%s ok.' % archive_name)
     finally:
         # setting back the original buildout.cfg file
         f = open(filename, 'w')
@@ -452,23 +560,59 @@ def deploy_release(path=None, target=None, archiving='full'):
         # removing MD5
         os.remove(join(root, 'MD5'))
 
-def copy_archives(from_=None, to=None):
+def copy_archives(from_, tarball, filter_eggs=['*'], clean=True):
     """Will copy downloads and eggs folders
     from a buildout to another.
 
     This will speed up creation time.
     """
-    if from_ is None and to is None:
-        if len(sys.argv) > 2:
-            from_ = sys.argv[1]
-            to = sys.argv[1]
+    if from_ is None and tarball is None:
+        display("Need both source and dest")
+        sys.exit(-1)
+
+    to = mkdtemp(prefix='releaser')
+
+    buildout = Buildout(from_, [])
+
+    buildout_directory = buildout['buildout']['directory']
+
+    for filename in glob(join(buildout_directory, '*.cfg')) + [join(buildout_directory, 'version.txt')]:
+        if os.path.isfile(filename):
+            shutil.copyfile(filename, filename.replace(buildout_directory, to))
         else:
-            sys.exit(1)
-    downloads = join(from_, 'downloads')
+            display('Warning: %s dos not exist' % filename)
+
+    downloads = buildout['buildout']['download-cache']
     downloads_target = join(to, 'downloads')
-    eggs = join(from_, 'eggs')
+    eggs = buildout['buildout']['eggs-directory']
     eggs_target = join(to, 'eggs')
 
+    egg_version = 'py' + '.'.join([str(v) for v in sys.version_info[:2]])
+
+    matches = set()
+
+    def match(path):
+        if '*' in filter_eggs:
+            return True
+        # check egg name. need (filter)-py(python-version)
+        egg_name = [p for p in path.split(os.sep) if p.endswith('.egg')]
+        if egg_name:
+            egg_name = egg_name[0]
+            for pattern in filter_eggs:
+                if fnmatch(egg_name, pattern) and egg_version in egg_name:
+                    if egg_name not in matches:
+                        matches.add(egg_name)
+                    return True
+        # check egg name. need (filter)-(gz|tar|zip)
+        if path.endswith('gz') or path.endswith('.zip') or path.endswith('tar'):
+            egg_name = os.path.basename(path)
+            for pattern in filter_eggs:
+                if fnmatch(egg_name, pattern):
+                    if egg_name not in matches:
+                        matches.add(egg_name)
+                    return True
+        return False
+
     for source, target in ((downloads, downloads_target),
                            (eggs, eggs_target)):
         if not os.path.exists(source):
@@ -481,29 +625,40 @@ def copy_archives(from_=None, to=None):
                 i += 1
             shutil.move(target, name)
 
-        _log('Copying %s to %s' % (source, target))
+        display('Copying %s to %s' % (source, target))
         os.mkdir(target)
+
         for root, dirs, filenames in os.walk(source):
             root_target = root.replace(source, target)
             if '.svn' in root:
                 continue
-            for dir_ in dirs:
-                if '.svn' in dir_:
-                    continue
-                fullpath = join(root_target, dir_)
-                if os.path.exists(fullpath) or '.svn' in fullpath:
-                    continue
-                os.mkdir(fullpath)
-
             for filename in filenames:
                 filename_source = join(root, filename)
                 filename_target = join(root_target, filename)
                 if os.path.exists(filename_target):
                     continue
-                shutil.copyfile(filename_source, filename_target)
+                if match(filename_source):
+                    dirname = os.path.dirname(filename_target)
+                    if not os.path.isdir(dirname):
+                        os.makedirs(dirname)
+                    shutil.copyfile(filename_source, filename_target)
+
+    buildout = join(to, 'buildout.cfg')
+    if from_ != 'buildout.cfg':
+        set_option(buildout, 'buildout', 'extends', from_)
+    set_option(buildout, 'buildout', 'install-from-cache', 'true')
+    set_option(buildout, 'buildout', 'offline', 'true')
+    set_option(buildout, 'buildout', 'newest', 'false')
+
+    archive_contents(tarball, to)
+    if clean:
+        shutil.rmtree(to)
+        return matches
+    else:
+        return to
 
 def console_build_md5(folder=None):
-    _log(build_md5(folder))
+    display(build_md5(folder))
 
 def build_md5(folder=None):
     """generate an MD5 stamp, by recursively reading
@@ -539,26 +694,58 @@ _easy_install_cmd = _safe_arg(
 
 def console_project_eggs(args=None):
     """for console calls"""
-    if args is None:
-        if len(sys.argv) == 1:
-            _log('Usage: %s configuration-file [tarball filters-eggs]]' % sys.argv[0])
-            sys.exit(1)
-        if len(sys.argv) > 1:
-            cfg = sys.argv[1]
-        if len(sys.argv) > 2:
-            tarball = sys.argv[2]
-        else:
-            tarball = None
-        if len(sys.argv) > 3:
-            filter_eggs = sys.argv[3].split(',')
-        else:
-            filter_eggs = ['iw.*']
+    usage = '%prog [options] [archive_name]'
+    parser = OptionParser(usage=usage)
+    parser.add_option("-c", "--config", dest="cfg",
+                      help="Config file. Default: buildout.cfg",
+                      default="buildout.cfg", metavar="CFG")
+    parser.add_option("--force", dest="force",
+                      help="Force to build archive without checking. Use it at your own risk !",
+                      default=False, action="store_true")
+    parser.add_option("--copy", dest="copy",
+                      help="Only copy eggs from the existing eggs-directory",
+                      default=False, action="store_true")
+    parser.add_option("-e", "--egg", dest="filter_eggs",
+                      help="Eggs name or pattern. more than one is allowed",
+                      default=[], action='append',
+                      metavar='EGG_FILTER')
+    options, args = parser.parse_args(args or sys.argv)
+
+    if not options.cfg or not os.path.isfile(options.cfg):
+        parser.parse_args(['-h'])
     else:
-        cfg, tarball, filter_eggs = args
-    _log('Scanning...')
-    eggs = project_eggs(cfg, tarball, filter_eggs)
-    _log('\n\nEggs collected:')
-    _log('\n'.join(['    - %s' % egg for egg in eggs]))
+        cfg = options.cfg
+
+    filter_eggs = options.filter_eggs
+
+    if len(args) != 2:
+        name = os.path.splitext(cfg)[0]
+        if os.path.isfile('version.txt'):
+            version = open('version.txt').read().strip()
+            tarball = '%s-%s-eggs.tar.gz' % (name, version)
+        else:
+            tarball = '%s-eggs.tar.gz' % name
+    else:
+        tarball = args[1]
+
+    buildout = Buildout(cfg, [])
+
+    if not options.force:
+        if not os.path.isfile('version.txt'):
+            parser.error('Seems you try to run this command from a non released buildout. version.txt not found')
+
+        if  buildout['buildout'].get('develop'):
+            parser.error("You can't use this command with a develop option")
+
+    display('Generating %s...' % tarball)
+    display('Scanning for %s...' % ', '.join(filter_eggs))
+    if options.copy:
+        eggs = copy_archives(cfg, tarball, filter_eggs, clean=True)
+    else:
+        eggs = project_eggs(cfg, tarball, filter_eggs)
+
+    display('\n\nEggs collected:')
+    display('\n'.join(['    - %s' % egg for egg in eggs]))
 
 class EggInstaller(Installer):
 
@@ -588,7 +775,7 @@ class EggInstaller(Installer):
         return dists
 
 @savewd
-def project_eggs(cfg, tarball=None, filter_eggs=None):
+def project_eggs(cfg, tarball=None, filter_eggs=None, verbose=False):
     """Scans for all eggs used by a buildout in eggs, and return their names.
 
     If install_folder is given, the eggs scanned are fetched and installed
@@ -618,7 +805,7 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
                 for l in lines.split('\n') if l.strip() != '']
 
     tmp = mkdtemp()
-    _log('Working in %s' % tmp)
+    display_step('Working in %s' % tmp, verbose)
 
     eggs_directory = join(tmp, 'eggs')
     os.mkdir(eggs_directory)
@@ -687,7 +874,8 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
 
     # let's collect the used eggs
     _eggs = []
-    for section in buildout:
+    sections = buildout.keys()
+    for section in sections:
         if section == 'versions':
             continue
         if 'eggs' in buildout[section]:
@@ -703,7 +891,7 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
             eggs.append(egg)
 
     # installing the eggs
-    _log('Installing eggs in %s.' % install_folder)
+    display_step('Installing eggs in %s.' % install_folder, verbose)
     installer = EggInstaller(eggs_directory, links=find_links,
                              index=index, versions=versions,
                              filter_eggs=filter_eggs)
@@ -716,10 +904,10 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
         if not _install(egg):
             continue
         try:
-            _log('Installing %s' % egg)
+            display_step('Installing %s' % egg, verbose)
             installer.install((egg,))
         except (MissingDistribution, DistutilsError):
-            _log('Could not install %s' % egg)
+            display_step('Could not install %s' % egg, verbose)
 
     def _check(egg):
         egg = egg.strip()
@@ -783,7 +971,7 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
     _copy(downloads_directory, join(install_folder, 'downloads'))
 
     # let's copy the .cfg files as well
-    _log('Copying all cfg files')
+    display_step('Copying all cfg files', verbose)
     for file_ in os.listdir(buildout_dir):
         if not os.path.splitext(file_)[-1] in ('.txt', '.cfg'):
             continue
@@ -805,7 +993,7 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
         shutil.copyfile(join(buildout_dir, file_), to_)
 
     # archiving
-    _log('Archiving to %s' % tarball)
+    display_step('Archiving to %s' % tarball, verbose)
     exclude = ['downloads', 'develop-eggs']
     archive_contents(tarball, install_folder, exclude=exclude)
     installed = ['%s (%s)' % (egg.split('-')[0], egg.split('-')[1])
@@ -821,7 +1009,7 @@ def console_make_svn_structure_for_package(package_directory=None):
     """
     if package_directory is None:
         if len(sys.argv) != 2:
-            _log('Usage: %s package_directory' % sys.argv[0])
+            display('Usage: %s package_directory' % sys.argv[0])
             sys.exit(1)
         package_directory = sys.argv[1]
 
@@ -833,24 +1021,142 @@ def console_make_svn_structure_for_package(package_directory=None):
     shutil.move(package_directory, tmp_name)
 
     #make the svn structure
-    _log("creating %s/tags" % package_directory)
+    display("creating %s/tags" % package_directory)
     os.makedirs("%s/tags" % package_directory)
 
-    _log("creating %s/branches" % package_directory)
+    display("creating %s/branches" % package_directory)
     os.mkdir("%s/branches" % package_directory)
 
-    _log("creating %s/trunk" % package_directory)
+    display("creating %s/trunk" % package_directory)
     shutil.move(tmp_name, "%s/trunk" % package_directory)
 
     #svn add and ci it, if the parent directory is a working copy
     if base.has_svn_support():
         try:
-
             base.svn_add(package_directory)
             base.svn_commit('initial import of %s' % package_directory)
-            _log("added %s to svn" % package_directory)
+            display("added %s to svn" % package_directory)
         except:
-            _log("Current directory is not a working copy. Skip adding %s to svn" % package_directory)
+            display("Current directory is not a working copy. Skip adding %s to svn" % package_directory)
 
-    _log('Done...')
+    display('Done...')
 
+def run_setup_release(args):
+    # run setup release at version
+
+    # this is ugly but multiprocessing seems to fail with the first Popen
+    try:
+        runwait('echo')
+    except:
+        pass
+
+    setup, ver = args
+    env = os.environ.copy()
+    env['PYTHONPATH'] = os.pathsep.join(sys.path)
+
+    dirname = os.path.dirname(setup)
+    os.chdir(dirname)
+    name = packet.get_name()
+    current_version = packet.get_version()
+    if ver != current_version:
+        command = '%s setup.py release -a --version=%s' % (sys.executable, ver)
+        exit_code, output, errors = runwait(command, env)
+    else:
+        exit_code = 0
+        output = '%s is already at version %s. skipping' % (name, ver)
+        errors = ''
+    return name, ver, setup, exit_code, output, errors
+
+@savewd
+def release_eggs(version=None, filter_eggs=[], prod_cfg='prod.cfg', dev_cfg='dev.cfg',
+                 interactive=True, verbose=False):
+    """release all eggs to a specific version
+    """
+    root = os.getcwd()
+
+    released = {}
+    release_eggs = []
+
+    if version is None:
+        # try to get a default version from the buildout version
+        if os.path.isfile('version.txt'):
+            default_version = open('version.txt').read().strip()
+        else:
+            default_version = None
+        parser = OptionParser()
+        parser.add_option("-v", "--version", dest="version",
+                          help="Version",
+                          default=default_version)
+        parser.add_option("-e", "--egg", dest="filter_eggs",
+                          help="Eggs name or pattern. more than one is allowed",
+                          default=[], action='append',
+                          metavar='EGG_FILTER')
+        parser.add_option("--verbose", dest="verbose",
+                          help="More print",
+                          action='store_true', default=verbose)
+        options, args = parser.parse_args()
+        version = options.version
+        verbose = options.verbose
+        if not version:
+            parser.error('Version is required')
+        filter_eggs = options.filter_eggs
+    else:
+        default_version = version
+
+    if os.path.isfile(dev_cfg) and os.path.isfile(prod_cfg):
+        # we are in a buildout project
+        buildout_dev = Buildout(dev_cfg, [])
+        buildout_prod = Buildout(prod_cfg, [])
+
+        fixed_versions = buildout_prod['versions']
+        versions = {}
+
+        develop_eggs = buildout_dev['buildout'].get('develop')
+        develop_eggs = [e.strip() for e in develop_eggs.split('\n')]
+        develop_eggs = [e for e in develop_eggs if e]
+        for develop in sorted(develop_eggs):
+            develop = os.path.abspath(develop)
+            if os.path.isfile(join(develop, 'setup.py')):
+                package = os.path.basename(develop)
+                ver = fixed_versions.get(package, default_version)
+                versions[develop] = ver
+
+        for dirname, ver in sorted(versions.items()):
+            display('- %s=%s' % (os.path.basename(dirname), ver))
+
+        if interactive:
+            reply = raw_input('Is your %s up to date ? [y/N] ' % prod_cfg)
+        else:
+            display('Is your %s up to date ? [Y/n] ' % prod_cfg)
+            reply = 'y'
+        if reply.lower() == 'y':
+            for dirname, ver in versions.items():
+                setup = join(dirname, 'setup.py')
+                if os.path.isfile(setup):
+                    release_eggs.append([setup, ver])
+                else:
+                    raise base.ReleaseError('No such file %s' % setup)
+
+    elif filter_eggs:
+        # we are in a develop directory
+        for pattern in filter_eggs:
+            for setup in glob(join(root, pattern, 'setup.py')):
+                release_eggs.append([setup, version])
+
+    if os.environ.get('DEBUG', 'false') == True:
+        results = [run_setup_release((setup, ver)) for setup, ver in release_eggs]
+    else:
+        pool = Pool(3)
+        results = pool.map(run_setup_release, release_eggs)
+
+
+    for result in results:
+        name, ver, setup, exit_code, output, errors = result
+        if exit_code != 0:
+            display(errors)
+            raise base.ReleaseError('Your release fail')
+        else:
+            display_step(output, verbose)
+            released[name] = ver
+
+    return released
