@@ -16,6 +16,9 @@
 # Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 import os, shutil
+
+HOME_DIR = os.path.join(os.path.dirname(__file__), 'tests')
+
 import iw.email.testing
 import zc.buildout.testing
 from zc.buildout.testing import system, rmdir, mkdir
@@ -77,6 +80,7 @@ def customBuildoutSetup(test):
     shutil.copyfile(source, target)
 
 def releaserSetUp(test):
+    save(HOME_DIR)
     customBuildoutSetup(test)
     test.globs['test'] = test
     test.globs['svn_start'] = svn_start
@@ -85,6 +89,7 @@ def releaserSetUp(test):
 def releaserTearDown(test):
     zc.buildout.testing.buildoutTearDown(test)
     iw.email.testing.smtpTearDown()
+    purge(HOME_DIR)
 
 def clearBuildout(test=None):
     zc.buildout.testing.buildoutTearDown(test)
@@ -111,3 +116,27 @@ def paster(cmd):
     runner = command(command_name)
     runner.run(args[1:])
 
+_snapshot = {}
+
+def save(directory):
+    elmts = []
+    for root, dirs, files in os.walk(directory):
+        for dir in dirs:
+            elmts.append(os.path.join(root, dir))
+        for file in files:
+            elmts.append(os.path.join(root, file))
+    _snapshot[directory] = elmts
+
+def purge(directory):
+    snapshot = _snapshot[directory]
+    for root, dirs, files in os.walk(directory):
+        for file in files:
+            path = os.path.join(root, file)
+            if path not in snapshot:
+                os.remove(path)
+        for dir in dirs:
+            path = os.path.join(root, dir)
+            if path not in snapshot:
+                shutil.rmtree(path, True)
+
+
